# 环境安装（首次运行前执行）
# pip install "unsloth[colab-new] @ git+https://github.com/unslothai/unsloth.git"
# pip install --no-deps "xformers<0.0.26" trl peft accelerate bitsandbytes
# pip install unsloth vllm
# pip install diffusers

from unsloth import FastLanguageModel
import torch
from datasets import load_dataset
from trl import SFTTrainer
from transformers import TrainingArguments

# 模型配置
model_name = "deepseek-ai/deepseek-r1"  # Hugging Face模型ID
max_seq_length = 2048  # 根据GPU内存调整
dtype = None  # None自动选择，或指定torch.float16/torch.bfloat16
load_in_4bit = True  # 4bit量化节省内存

# 加载模型
model, tokenizer = FastLanguageModel.from_pretrained(
    model_name=model_name,
    max_seq_length=max_seq_length,
    dtype=dtype,
    load_in_4bit=load_in_4bit,
)

# 添加LoRA适配器（参数高效微调）
model = FastLanguageModel.get_peft_model(
    model,
    r=16,  # LoRA秩
    target_modules=["q_proj", "k_proj", "v_proj", "o_proj"],
    lora_alpha=16,
    lora_dropout=0,
    bias="none",
    use_gradient_checkpointing=True,
)

# 准备数据集示例（需替换为实际数据）
dataset = load_dataset("json", data_files="your_dataset.json", split="train")

def formatting_prompts_func(examples):
    # 根据DeepSeek的指令格式构建prompt
    instructions = examples["instruction"]
    inputs = examples["input"]
    outputs = examples["output"]
    
    texts = []
    for instruction, input, output in zip(instructions, inputs, outputs):
        text = f"### Instruction:\n{instruction}\n\n### Input:\n{input}\n\n### Response:\n{output}"
        texts.append(text)
    return {"text" : texts}

dataset = dataset.map(formatting_prompts_func, batched=True)

# 训练参数配置
trainer = SFTTrainer(
    model=model,
    tokenizer=tokenizer,
    train_dataset=dataset,
    dataset_text_field="text",
    max_seq_length=max_seq_length,
    dataset_num_proc=2,
    args=TrainingArguments(
        per_device_train_batch_size=2,
        gradient_accumulation_steps=4,
        warmup_steps=5,
        max_steps=100,
        learning_rate=2e-5,
        fp16=not torch.cuda.is_bf16_supported(),
        bf16=torch.cuda.is_bf16_supported(),
        logging_steps=1,
        optim="adamw_8bit",
        weight_decay=0.01,
        lr_scheduler_type="linear",
        seed=42,
        output_dir="outputs",
    ),
)

# 执行训练（启用Unsloth优化）
trainer.train()

# 保存适配器权重
model.save_pretrained("lora_adapter")

# 推理示例
FastLanguageModel.for_inference(model)  # 启用优化推理模式
inputs = tokenizer(
    "### Instruction:\n解释量子计算的基本原理\n\n### Response:",
    return_tensors="pt",
).to("cuda")

outputs = model.generate(
    **inputs,
    max_new_tokens=256,
    temperature=0.7,
    do_sample=True,
)
print(tokenizer.decode(outputs[0], skip_special_tokens=True))