# model_runner.py
import torch
import time
import psutil  # 新增系统监控
from transformers import AutoTokenizer, AutoModelForCausalLM, TextStreamer
from src.config import Config
import logging
from typing import Union, List, Tuple

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

class ModelRunner:
    def __init__(self):
        self.device = Config.DEVICE
        self.modelId = Config.RUN_MODEL_DIR
        self._init_resources()
        self._load_model()
        self._init_streamer()

    def _init_resources(self):
        """初始化资源监控基线"""
        self.process = psutil.Process()
        self.initial_cpu_mem = self.process.memory_info().rss / 1024**3  # GB
        if torch.cuda.is_available():
            torch.cuda.reset_peak_memory_stats()
            self.initial_gpu_mem = torch.cuda.memory_allocated() / 1024**3  # GB

    def _load_model(self):
        """加载模型和分词器"""
        try:
            logger.info(f"Loading model from {self.modelId} on {self.device}")
            
            self.tokenizer = AutoTokenizer.from_pretrained(
                self.modelId,
                padding_side="left" if Config.BATCH_ENABLED else "right"
            )
            
            # 自动设备映射优化
            self.model = AutoModelForCausalLM.from_pretrained(
                self.modelId,
                torch_dtype=Config.TORCH_DTYPE,
                device_map="auto" if Config.DEVICE_STR == "cuda" else None,
                **Config.MODEL_LOAD_KWARGS
            )

            # 多GPU并行
            if Config.DEVICE_STR == "cuda" and torch.cuda.device_count() > 1:
                self.model = torch.nn.DataParallel(self.model)
                logger.info(f"Parallelized across {torch.cuda.device_count()} GPUs")

            # 分词器配置
            if self.tokenizer.pad_token is None:
                self.tokenizer.pad_token = self.tokenizer.eos_token

            self.model.eval()
            logger.info("Model loaded in evaluation mode")

        except Exception as e:
            logger.error(f"Model loading failed: {str(e)}")
            raise

    def _init_streamer(self):
        """初始化流式输出组件"""
        self.streamer = TextStreamer(
            self.tokenizer,
            skip_prompt=True,
            skip_special_tokens=True
        ) if Config.STREAMING else None

    def _log_resource_usage(self):
        """记录资源使用情况"""
        metrics = {}
        # CPU内存
        metrics["cpu_mem"] = self.process.memory_info().rss / 1024**3 - self.initial_cpu_mem
        
        # GPU内存
        if torch.cuda.is_available():
            metrics["gpu_mem"] = (
                torch.cuda.max_memory_allocated() / 1024**3 - self.initial_gpu_mem
            )
        
        logger.debug(f"Resource Usage: {metrics}")

    def generate(
        self, 
        prompts: Union[str, List[str]],
        **generation_kwargs
    ) -> Tuple[Union[str, List[str]], dict]:
        """
        支持单条/批量文本生成
        :param prompts: 输入文本或文本列表
        :return: (生成文本, 性能指标)
        """
        metrics = {"total_tokens": 0}
        try:
            total_start = time.perf_counter()
            
            # 批处理预处理
            is_batch = isinstance(prompts, list)
            inputs = self._preprocess(prompts, metrics)
            
            # 生成参数
            gen_params = self._build_generation_params(inputs, generation_kwargs)
            
            # 执行生成
            outputs = self._execute_generation(gen_params, metrics)
            
            # 后处理
            results = self._postprocess(outputs, inputs, is_batch, metrics)
            
            # 最终指标计算
            metrics["total_time"] = time.perf_counter() - total_start
            metrics["throughput"] = metrics["total_tokens"] / metrics["total_time"]
            
            self._log_performance(metrics)
            self._log_resource_usage()
            
            return results, metrics

        except Exception as e:
            logger.error(f"Generation error: {str(e)}")
            self._log_resource_usage()
            raise

    def _preprocess(self, prompts: Union[str, List[str]], metrics: dict):
        """预处理输入"""
        start = time.perf_counter()
        
        inputs = self.tokenizer(
            prompts,
            return_tensors="pt",
            padding=Config.BATCH_ENABLED,
            truncation=True,
            max_length=Config.MAX_INPUT_LENGTH
        ).to(self.device)
        
        metrics["preprocess_time"] = time.perf_counter() - start
        return inputs

    def _build_generation_params(self, inputs, generation_kwargs):
        """构建生成参数"""
        params = {
            "input_ids": inputs.input_ids,
            "attention_mask": inputs.attention_mask,
            "max_new_tokens": generation_kwargs.get("max_new_tokens", Config.MAX_NEW_TOKENS),
            "temperature": generation_kwargs.get("temperature", Config.TEMPERATURE),
            "top_p": generation_kwargs.get("top_p", Config.TOP_P),
            "num_beams": generation_kwargs.get("num_beams", Config.NUM_BEAMS),
            "do_sample": generation_kwargs.get("do_sample", Config.DO_SAMPLE),
            "pad_token_id": self.tokenizer.pad_token_id,
            "streamer": self.streamer if Config.STREAMING else None,
        }
        return {k: v for k, v in params.items() if v is not None}

    def _execute_generation(self, gen_params, metrics):
        """执行生成并记录指标"""
        start = time.perf_counter()
        
        with torch.inference_mode():
            outputs = self.model.generate(**gen_params)
        
        metrics["inference_time"] = time.perf_counter() - start
        metrics["total_tokens"] = outputs.shape[-1] - gen_params["input_ids"].shape[-1]
        return outputs

    def _postprocess(self, outputs, inputs, is_batch, metrics):
        """后处理输出"""
        start = time.perf_counter()
        
        # 批量解码
        results = []
        for i in range(outputs.shape[0]):
            # 跳过输入部分
            generated = outputs[i, inputs.input_ids.shape[1]:]
            try:
                text = self.tokenizer.decode(
                    generated,
                    skip_special_tokens=True
                )
                results.append(text.strip())
            except Exception as e:
                logger.warning(f"Decoding error for sample {i}: {str(e)}")
                results.append("")
        
        metrics["postprocess_time"] = time.perf_counter() - start
        return results if is_batch else results[0]

    def _log_performance(self, metrics):
        """记录性能日志"""
        log_msg = (
            f"Performance Summary || "
            f"Total: {metrics['total_time']:.2f}s | "
            f"Inference: {metrics['inference_time']:.2f}s | "
            f"Tokens: {metrics['total_tokens']} | "
            f"Speed: {metrics['throughput']:.1f} tok/s"
        )
        if Config.BATCH_ENABLED:
            log_msg += f" | Batch size: {metrics.get('batch_size', 1)}"
        logger.info(log_msg)

# 配置文件示例（src/config.py）
    
if __name__ == "__main__":
    # 使用示例
    runner = ModelRunner()
    
    # 单条生成
    single_prompt = "DeepSeek-R1的主要技术优势是"
    text, metrics = runner.generate(single_prompt)
    print(f"\nSingle Generation:\n{text}")
    
    # 批量生成
    batch_prompts = [
        "解释量子计算的基本原理",
        "写一首关于人工智能的诗",
        "如何快速学习PyTorch？"
    ]
    texts, metrics = runner.generate(batch_prompts)
    print("\nBatch Results:")
    for i, text in enumerate(texts):
        print(f"{i+1}. {text[:100]}...")
