#  model_runner.py
import torch
from torch.quantization import quantize_dynamic
from transformers import AutoTokenizer, AutoModelForCausalLM
from src.config import Config
import logging


logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

class ModelRunner:
    def __init__(self):
        # 修正设备设置
        self.device =Config.DEVICE # torch.device("cuda" if torch.cuda.is_available() else "cpu")
        logger.info(f"Using device: {self.device}")
        
        self.modelId=Config.RUN_MODEL_DIR
        logger.info(f"model id: {self.modelId}")

        try:
            # 加载训练后的模型和分词器
            self.tokenizer = AutoTokenizer.from_pretrained(self.modelId)
            self.model = AutoModelForCausalLM.from_pretrained(self.modelId,torch_dtype= Config.TORCH_DTYPE)
            # self.model = AutoModelForCausalLM.from_pretrained(self.modelId,torch_dtype= Config.TORCH_DTYPE,device_map="auto")
            
            if Config.DEVICE_STR=="cuda" and torch.cuda.device_count() > 1:
               self.model = torch.nn.DataParallel(self.model)  # 多 GPU 并行

            # 确保分词器有 pad_token
            if self.tokenizer.pad_token is None:
                self.tokenizer.pad_token = self.tokenizer.eos_token
                        
            # 将模型移动到设备
            self.model.to(self.device)
            logger.info("Trained model loaded successfully.")
        except Exception as e:
            logger.error(f"Failed to load model: {e}")
            raise

    def generate_text(self, prompt, max_length=None, temperature=None, do_sample=True):
        try:
            self.model.eval()
            
            # 编码输入并生成 attention_mask
            inputs = self.tokenizer(
                prompt, 
                return_tensors="pt", 
                padding=True, 
                truncation=True
            ).to(self.device)
            
            # 设置生成参数
            max_length = max_length or Config.MAX_TOKENS
            max_new_tokens = max_length or Config.MAX_NEW_TOKENS
            temperature = temperature or Config.TEMPERATURE

            # 生成文本
            outputs = self.model.generate(
                inputs.input_ids,
                attention_mask=inputs.attention_mask,  # 显式传递 attention_mask
                max_new_tokens=max_new_tokens,
                temperature=temperature,
                top_k=Config.TOP_K,
                top_p=Config.TOP_P,
                num_beams=Config.NUM_BEAMS,
                do_sample=do_sample,
                pad_token_id=self.tokenizer.pad_token_id  # 显式设置 pad_token_id
            )
            
            # 解码生成结果
            generated_text = self.tokenizer.decode(outputs[0], skip_special_tokens=True)
            return generated_text
        except Exception as e:
            logger.error(f"Generation failed: {e}")
            raise