# model_trainer.py
import torch
from torch.utils.data import Dataset, DataLoader
from transformers import AutoTokenizer, AutoModelForCausalLM, AdamW
from src.config import Config
import logging
import os

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

class TextDataset(Dataset):
    """自定义数据集类"""
    def __init__(self, file_path, tokenizer, seq_length):
        self.tokenizer = tokenizer
        self.seq_length = seq_length
        with open(file_path, "r", encoding="utf-8") as f:
            self.lines = f.read().splitlines()

    def __len__(self):
        return len(self.lines)

    def __getitem__(self, idx):
        line = self.lines[idx]
        tokens = self.tokenizer.encode(
            line, 
            max_length=self.seq_length, 
            truncation=True, 
            padding="max_length"
        )
        return torch.tensor(tokens)  # 返回 CPU 上的张量

class ModelTrainer:
    def __init__(self):
        """初始化模型、分词器和优化器"""
        self.device = Config.DEVICE
        logger.info(f"Using device: {self.device}")

        try:
            # 加载分词器和模型
            self.tokenizer = AutoTokenizer.from_pretrained(Config.PRETRAINED_MODEL_DIR)
            self.model = AutoModelForCausalLM.from_pretrained(Config.PRETRAINED_MODEL_DIR)
            self.model.to(self.device)  # 将模型移动到设备
            logger.info("Pretrained model and tokenizer loaded.")
        except Exception as e:
            logger.error(f"Failed to load pretrained model: {e}")
            raise

    def train(self):
        """训练模型"""
        try:
            # 加载数据集
            dataset = TextDataset(Config.PROCESSED_DATA_PATH, self.tokenizer, Config.SEQ_LENGTH)
            dataloader = DataLoader(dataset, batch_size=Config.BATCH_SIZE, shuffle=True, num_workers=Config.NUM_WORKERS)

            # 初始化优化器
            optimizer = AdamW(self.model.parameters(), lr=Config.LEARNING_RATE)

            # 训练循环
            self.model.train()
            for epoch in range(Config.EPOCHS):
                for batch in dataloader:
                    # 将输入数据移动到设备
                    inputs = batch.to(self.device)
                    
                    # 前向传播
                    outputs = self.model(inputs, labels=inputs)
                    loss = outputs.loss
                    
                    # 反向传播和优化
                    loss.backward()
                    optimizer.step()
                    optimizer.zero_grad()

                logger.info(f"Epoch {epoch + 1}/{Config.EPOCHS}, Loss: {loss.item()}")

            # 保存训练后的模型
            os.makedirs(Config.TRAINED_MODEL_DIR, exist_ok=True)
            self.model.save_pretrained(Config.TRAINED_MODEL_DIR)
            self.tokenizer.save_pretrained(Config.TRAINED_MODEL_DIR)
            logger.info(f"Model saved to {Config.TRAINED_MODEL_DIR}")

        except Exception as e:
            logger.error(f"Training failed: {e}")
            raise
