import torch
import torch.distributed as dist
from torch.utils.data import Dataset, DataLoader
from torch.nn.parallel import DistributedDataParallel as DDP
from torch.utils.data.distributed import DistributedSampler
from transformers import AutoTokenizer, AutoModelForCausalLM, AdamW
from src.config import Config
import logging
import os

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

class TextDataset(Dataset):
    """自定义数据集类"""
    def __init__(self, file_path, tokenizer, seq_length):
        self.tokenizer = tokenizer
        self.seq_length = seq_length
        with open(file_path, "r", encoding="utf-8") as f:
            self.lines = f.read().splitlines()

    def __len__(self):
        return len(self.lines)

    def __getitem__(self, idx):
        line = self.lines[idx]
        tokens = self.tokenizer.encode(
            line, 
            max_length=self.seq_length, 
            truncation=True, 
            padding="max_length"
        )
        return torch.tensor(tokens)  # 返回 CPU 上的张量

class ModelTrainer:
    def __init__(self, local_rank, world_size):
        """初始化模型、分词器和优化器"""
        self.local_rank = local_rank
        self.world_size = world_size
        self.device = torch.device(f"cuda:{local_rank}" if torch.cuda.is_available() else "cpu")
        logger.info(f"Using device: {self.device}")

        try:
            # 加载分词器和模型
            self.tokenizer = AutoTokenizer.from_pretrained(Config.PRETRAINED_MODEL_DIR)
            self.model = AutoModelForCausalLM.from_pretrained(Config.PRETRAINED_MODEL_DIR)
            self.model.to(self.device)  # 将模型移动到设备

            # 使用 DistributedDataParallel 包装模型
            self.model = DDP(self.model, device_ids=[local_rank], output_device=local_rank)
            logger.info("Pretrained model and tokenizer loaded.")
        except Exception as e:
            logger.error(f"Failed to load pretrained model: {e}")
            raise

    def train(self):
        """训练模型"""
        try:
            # 加载数据集
            dataset = TextDataset(Config.PROCESSED_DATA_PATH, self.tokenizer, Config.SEQ_LENGTH)
            sampler = DistributedSampler(dataset, num_replicas=self.world_size, rank=self.local_rank)
            dataloader = DataLoader(
                dataset, 
                batch_size=Config.BATCH_SIZE, 
                sampler=sampler, 
                num_workers=4
            )

            # 初始化优化器
            optimizer = AdamW(self.model.parameters(), lr=Config.LEARNING_RATE)

            # 训练循环
            self.model.train()
            for epoch in range(Config.EPOCHS):
                sampler.set_epoch(epoch)  # 设置 epoch 以打乱数据
                for batch in dataloader:
                    # 将输入数据移动到设备
                    inputs = batch.to(self.device)
                    
                    # 前向传播
                    outputs = self.model(inputs, labels=inputs)
                    loss = outputs.loss
                    
                    # 反向传播和优化
                    loss.backward()
                    optimizer.step()
                    optimizer.zero_grad()

                if self.local_rank == 0:  # 仅主进程记录日志
                    logger.info(f"Epoch {epoch + 1}/{Config.EPOCHS}, Loss: {loss.item()}")

            # 保存训练后的模型（仅主进程保存）
            if self.local_rank == 0:
                os.makedirs(Config.TRAINED_MODEL_DIR, exist_ok=True)
                self.model.module.save_pretrained(Config.TRAINED_MODEL_DIR)
                self.tokenizer.save_pretrained(Config.TRAINED_MODEL_DIR)
                logger.info(f"Model saved to {Config.TRAINED_MODEL_DIR}")

        except Exception as e:
            logger.error(f"Training failed: {e}")
            raise

def setup_distributed():
    """初始化分布式训练环境"""
    dist.init_process_group(backend="nccl")
    local_rank = int(os.environ["LOCAL_RANK"])
    world_size = int(os.environ["WORLD_SIZE"])
    return local_rank, world_size

def cleanup_distributed():
    """清理分布式训练环境"""
    dist.destroy_process_group()

def main():
    # 初始化分布式训练
    local_rank, world_size = setup_distributed()

    try:
        # 初始化训练器
        trainer = ModelTrainer(local_rank, world_size)
        trainer.train()
    finally:
        # 清理分布式训练环境
        cleanup_distributed()

if __name__ == "__main__":
    main()